@@ -11,10 +11,12 @@
 ########################################################################
 
 """Test module for queries on datasets."""
+from __future__ import absolute_import
 
 import re
 import sys
 import types
+import warnings
 import functools
 
 import numpy
@@ -26,6 +28,12 @@ from tables.tests.common import unittest
 from tables.tests.common import verbosePrint as vprint
 from tables.tests.common import PyTablesTestCase as TestCase
 
+import six
+from six.moves import range
+
+from numpy import (log10, exp, log, abs, sqrt, sin, cos, tan,
+                   arcsin, arccos, arctan)
+
 
 # Data parameters
 # ---------------
@@ -54,9 +62,9 @@ type_info = {
     'int16': (numpy.int16, int),
     'uint16': (numpy.uint16, int),
     'int32': (numpy.int32, int),
-    'uint32': (numpy.uint32, long),
-    'int64': (numpy.int64, long),
-    'uint64': (numpy.uint64, long),
+    'uint32': (numpy.uint32, int),
+    'int64': (numpy.int64, int),
+    'uint64': (numpy.uint64, int),
     'float32': (numpy.float32, float),
     'float64': (numpy.float64, float),
     'complex64': (numpy.complex64, complex),
@@ -68,6 +76,14 @@ type_info = {
 }
 """NumPy and Numexpr type for each PyTables type that will be tested."""
 
+# globals dict for eval()
+func_info = {'log10': log10, 'log': log, 'exp': exp,
+             'abs': abs, 'sqrt': sqrt,
+             'sin': sin, 'cos': cos, 'tan': tan,
+             'arcsin': arcsin, 'arccos': arccos, 'arctan': arctan}
+"""functions and NumPy.ufunc() for each function that will be tested."""
+
+
 if hasattr(numpy, 'float16'):
     type_info['float16'] = (numpy.float16, float)
 # if hasattr(numpy, 'float96'):
@@ -80,10 +96,10 @@ if hasattr(numpy, 'float16'):
 #    type_info['complex256'] = (numpy.complex256, complex)
 
 sctype_from_type = dict((type_, info[0])
-                        for (type_, info) in type_info.iteritems())
+                        for (type_, info) in six.iteritems(type_info))
 """Maps PyTables types to NumPy scalar types."""
 nxtype_from_type = dict((type_, info[1])
-                        for (type_, info) in type_info.iteritems())
+                        for (type_, info) in six.iteritems(type_info))
 """Maps PyTables types to Numexpr types."""
 
 heavy_types = frozenset(['uint8', 'int16', 'uint16', 'float32', 'complex64'])
@@ -103,7 +119,7 @@ def append_columns(classdict, shape=()):
 
     """
     heavy = common.heavy
-    for (itype, type_) in enumerate(sorted(type_info.iterkeys())):
+    for (itype, type_) in enumerate(sorted(six.iterkeys(type_info))):
         if not heavy and type_ in heavy_types:
             continue  # skip heavy type in non-heavy mode
         colpos = itype + 1
@@ -200,9 +216,9 @@ def fill_table(table, shape, nrows):
     size = int(numpy.prod(shape, dtype=SizeType))
 
     row, value = table.row, 0
-    for nrow in xrange(nrows):
+    for nrow in range(nrows):
         data = numpy.arange(value, value + size).reshape(shape)
-        for (type_, sctype) in sctype_from_type.iteritems():
+        for (type_, sctype) in six.iteritems(sctype_from_type):
             if not heavy and type_ in heavy_types:
                 continue  # skip heavy type in non-heavy mode
             colname = 'c_%s' % type_
@@ -313,6 +329,8 @@ left_bound = row_period // 4
 """Operand of left side operator in comparisons with operator pairs."""
 right_bound = row_period * 3 // 4
 """Operand of right side operator in comparisons with operator pairs."""
+func_bound = 0.8  # must be <1 for trig functions to be able to fail
+"""Operand of right side operator in comparisons with functions. """
 extra_conditions = [
     '',                     # uses one index
     '& ((c_extra + 1) < 0)',  # uses one index
@@ -335,14 +353,15 @@ class TableDataTestCase(BaseTableQueryTestCase):
     _testfmt_heavy = 'test_h%04d'
 
 
-def create_test_method(type_, op, extracond):
+def create_test_method(type_, op, extracond, func=None):
     sctype = sctype_from_type[type_]
 
     # Compute the value of bounds.
     condvars = {'bound': right_bound,
                 'lbound': left_bound,
-                'rbound': right_bound}
-    for (bname, bvalue) in condvars.iteritems():
+                'rbound': right_bound,
+                'func_bound': func_bound}
+    for (bname, bvalue) in six.iteritems(condvars):
         if type_ == 'string':
             bvalue = str_format % bvalue
         bvalue = nxtype_from_type[type_](bvalue)
@@ -357,12 +376,14 @@ def create_test_method(type_, op, extracond):
         cond = colname
     elif op == '~':  # unary
         cond = '~(%s)' % colname
-    elif op == '<':  # binary variable-constant
+    elif op == '<' and func is None:  # binary variable-constant
         cond = '%s %s %s' % (colname, op, repr(condvars['bound']))
     elif isinstance(op, tuple):  # double binary variable-constant
         cond = ('(lbound %s %s) & (%s %s rbound)'
                 % (op[0], colname, colname, op[1]))
-    else:  # binary variable-variable
+    elif func is not None:
+        cond = '%s(%s) %s func_bound' % (func, colname, op)
+    else:  # function or binary variable-variable
         cond = '%s %s bound' % (colname, op)
     if extracond:
         cond = '(%s) %s' % (cond, extracond)
@@ -407,7 +428,12 @@ def create_test_method(type_, op, extracond):
                 pyvars['c_extra'] = row['c_extra']
                 pyvars['c_idxextra'] = row['c_idxextra']
                 try:
-                    isvalidrow = eval(pycond, {}, pyvars)
+                    with warnings.catch_warnings():
+                        warnings.filterwarnings(
+                            'ignore',
+                            'invalid value encountered in arc(cos|sin)',
+                            RuntimeWarning)
+                        isvalidrow = eval(pycond, func_info, pyvars)
                 except TypeError:
                     raise SilentlySkipTest(
                         "The Python type does not support the operation.")
@@ -463,33 +489,42 @@ def create_test_method(type_, op, extracond):
     test_method.__doc__ = "Testing ``%s``." % cond
     return test_method
 
+
+def add_test_method(type_, op, extracond='', func=None):
+    global testn
+    # Decide to which set the test belongs.
+    heavy = type_ in heavy_types or op in heavy_operators
+    if heavy:
+        testfmt = TableDataTestCase._testfmt_heavy
+        numfmt = ' [#H%d]'
+    else:
+        testfmt = TableDataTestCase._testfmt_light
+        numfmt = ' [#L%d]'
+    tmethod = create_test_method(type_, op, extracond, func)
+    # The test number is appended to the docstring to help
+    # identify failing methods in non-verbose mode.
+    tmethod.__name__ = testfmt % testn
+    # tmethod.__doc__ += numfmt % testn
+    tmethod.__doc__ += testfmt % testn
+    if sys.version_info[0] < 3:
+        imethod = types.MethodType(tmethod, None, TableDataTestCase)
+    else:
+        imethod = tmethod
+    setattr(TableDataTestCase, tmethod.__name__, imethod)
+    testn += 1
+
 # Create individual tests.  You may restrict which tests are generated
 # by replacing the sequences in the ``for`` statements.  For instance:
 testn = 0
 for type_ in type_info:  # for type_ in ['string']:
     for op in operators:  # for op in ['!=']:
-        # Decide to which set the test belongs.
-        heavy = type_ in heavy_types or op in heavy_operators
-        if heavy:
-            testfmt = TableDataTestCase._testfmt_heavy
-            numfmt = ' [#H%d]'
-        else:
-            testfmt = TableDataTestCase._testfmt_light
-            numfmt = ' [#L%d]'
         for extracond in extra_conditions:  # for extracond in ['']:
-            tmethod = create_test_method(type_, op, extracond)
-            # The test number is appended to the docstring to help
-            # identify failing methods in non-verbose mode.
-            tmethod.__name__ = testfmt % testn
-            # tmethod.__doc__ += numfmt % testn
-            tmethod.__doc__ += testfmt % testn
-            if sys.version_info[0] < 3:
-                imethod = types.MethodType(tmethod, None, TableDataTestCase)
-            else:
-                imethod = tmethod
-            setattr(TableDataTestCase, tmethod.__name__, imethod)
-            testn += 1
+            add_test_method(type_, op, extracond)
 
+for type_ in ['float32', 'float64']:
+    for func in func_info:  # i for func in ['log10']:
+        for op in operators:
+            add_test_method(type_, op, func=func)
 
 # Base classes for non-indexed queries.
 NX_BLOCK_SIZE1 = 128  # from ``interpreter.c`` in Numexpr
